﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Testing;

namespace OpcLabs.UnitTestLib.Testers
{
    public class XamlSerializationTester : Tester
    {
        public XamlSerializationTester()
        {
            TestCategories.Add("Serialization");
            TestCategories.Add("XamlSerialization");
        }

        public XamlSerializationTester([NotNull] Action<object, object> assertion)
            : this()
        {
            Assertion = assertion;
        }

        [CanBeNull]
        public Action<object, object> Assertion { get; }

        protected override void Initialize(Type typeUnderTest)
        {
            base.Initialize(typeUnderTest);

            // ReSharper disable once LoopCanBePartlyConvertedToQuery
            foreach (KeyValuePair<string, CreatorEntry<InstancesAttribute>> instanceCreator in InstanceCreators)
            {
                string instanceName = instanceCreator.Key;
                Debug.Assert(instanceName != null);
                AddTestMethod(CreateRoundtripTestMethod(instanceName));
            }
        }

        [NotNull]
        private TestMethod CreateRoundtripTestMethod([NotNull] string instanceName)
        {
            return new DelegatedTestMethod(
                string.Format(CultureInfo.InvariantCulture, "Roundtrip_{0}", instanceName), 
                () =>
                    {
                        object instance = CreateInstance(instanceName);
#if (NETFRAMEWORK)
                        if (Assertion != null)
                            Tests.XamlSerializationTest(instance, Assertion);
                        else
                            Tests.XamlSerializationTest(instance);
#else
                        AssertUtilities.Inconclusive("Cannot currently test XAML serialization under .NET Standard.");
#endif
                    });
        }
    }
}
