﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Collections.Specialized;
using OpcLabs.BaseLib.OperationModel;

namespace OpcLabs.BaseLibTest.OperationModel
{
    public partial class OperationResultTest
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Formatting_F_Failure_ErrorId_Test()
        {
            Assert.AreEqual("*** Failure -2: Some error [...] (0 diagnostics)", new OperationResult { ErrorMessage = "Some error\nSecond line", ErrorId = "-2" }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Failure_NoErrorId_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...] (0 diagnostics)", new OperationResult { ErrorMessage = "Some error\nSecond line" }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success:  (4 diagnostics)",
                new OperationResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList())
                }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_ErrorId_Test()
        {
            Assert.AreEqual("Success:  (0 diagnostics)", new OperationResult { ErrorId = "3" }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success:  (0 diagnostics)", new OperationResult().ToString("F"));
        }

        [TestMethod]
        public void Formatting_g_Failure_ErrorId_Test()
        {
            Assert.AreEqual("*** Failure -2: Some error [...]", new OperationResult { ErrorMessage = "Some error\nSecond line", ErrorId = "-2" }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Failure_NoErrorId_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new OperationResult { ErrorMessage = "Some error\nSecond line" }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success",
                new OperationResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList())
                }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_ErrorId_Test()
        {
            Assert.AreEqual("Success", new OperationResult { ErrorId = "3" }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success", new OperationResult().ToString("g"));
        }

        [TestMethod]
        public void ToString_Failure_ErrorId_Test()
        {
            Assert.AreEqual("*** Failure -2: Some error [...]", new OperationResult { ErrorMessage = "Some error\nSecond line", ErrorId = "-2" }.ToString());
        }

        [TestMethod]
        public void ToString_Failure_NoErrorId_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new OperationResult { ErrorMessage = "Some error\nSecond line" }.ToString());
        }

        [TestMethod]
        public void ToString_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success (4 diagnostics)",
                new OperationResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList())
                }.ToString());
        }

        [TestMethod]
        public void ToString_Success_ErrorId_Test()
        {
            Assert.AreEqual("Success", new OperationResult { ErrorId = "3" }.ToString());
        }

        [TestMethod]
        public void ToString_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success", new OperationResult().ToString());
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
