﻿using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.DataAccess.LiveMapping;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.LiveMapping._Testers
{
    class ItemIdTemplateTester
    {
        private class TestRecord
        {
            public int Count { get; set; }
        }

        [DAItemIdTemplate("$(ParentItemId).$(BrowseName)"), DAType, DASubscription(RequestedUpdateRate = 100)]
        private class ItemIdTemplateTestSimulation
        {
            public ItemIdTemplateTestSimulation([NotNull] TestRecord testRecord)
            {
                TestRecord = testRecord;
            }

            [DANode, DAItem]
            public double Random
            {
                set
                {
                    TestRecord.Count++;
                    AssertUtilities.WriteLine(value);
                }
            }

            [NotNull]
            // ReSharper disable MemberCanBePrivate.Local
            public TestRecord TestRecord { get; }
            // ReSharper restore MemberCanBePrivate.Local
        }

        [DAItemIdTemplate("$(BrowseName)"), DAType]
        private class ItemIdTemplateTestRoot
        {
            public ItemIdTemplateTestRoot([NotNull] TestRecord testRecord)
            {
                Simulation = new ItemIdTemplateTestSimulation(testRecord);
            }

            [DANode]
            public ItemIdTemplateTestSimulation Simulation { get; private set; }
        }

        public void Test()
        {
            const int timeout = 10 * 1000;

            var testRecord = new TestRecord();

            var clientMapper = new DAClientMapper();
            var testTarget = new ItemIdTemplateTestRoot(testRecord);

            clientMapper.Map(testTarget, new DAMappingContext { ServerDescriptor = LegacyTestDefs.ProgIds.KitServer });

            clientMapper.Subscribe(true);
            AssertUtilities.WaitingPhase(timeout);
            clientMapper.Subscribe(false);

            Assert.IsTrue(testRecord.Count >= 2);
        }
    }
}
