﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.PubSub;
using OpcLabs.EasyOpc.UA.PubSub.Configuration;
using OpcLabs.EasyOpc.UA.PubSub.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.OpcInteroperabilityTest.UA.PubSub.Publishers
{
    [TestClass]
    public class SoftingIndustrialAutomation_dataFeedNetStandardPubSubSdk_Test
    {
        [TestMethod]
        public void FixedLayoutTest()
        {
            UAPubSubConnectionDescriptor connectionDescriptor = "opc.udp://239.0.0.1";
            UASubscribeDataSetFilter subscribeDataSetFilter =
                UASubscribeDataSetFilter.Create(UAPublisherIdType.UInt16, 10);

            //connectionDescriptor.UseCaptureDeviceUdpMessageChannel(
            //    @"C:\Users\ZbynekZahradnik\Desktop\iopws 20190701\withsoftingfixedanddynamic.pcap");

            var subscriber = new EasyUASubscriber();
            subscriber.DataSetMessage += (sender, args) =>
            {
                AssertUtilities.WriteLine(args);
            };

            // 

            var dataSetMetaData1 = new UADataSetMetaData("Simple", new UAFieldMetaDataCollection
                {
                    new UAFieldMetaData("Toggle", UABuiltInType.Boolean),
                    new UAFieldMetaData("Int32", UABuiltInType.Int32),
                    new UAFieldMetaData("Int32Fast", UABuiltInType.Int32),
                    new UAFieldMetaData("DateTime", UABuiltInType.DateTime)
                });
            var subscribeDataSetArguments1 = new EasyUASubscribeDataSetArguments(
                connectionDescriptor, subscribeDataSetFilter, dataSetMetaData1)
            {
                DataSetSubscriptionDescriptor = { CommunicationParameters =
                    {
                        MessageReceiveTimeout = 6000,
                        UadpDataSetReaderMessageParameters = { DataSetOffset = 15 }
                    }}
            };

            //

            var dataSetMetaData2 = new UADataSetMetaData("AllTypes", new UAFieldMetaDataCollection
            {
                new UAFieldMetaData("BoolToggle", UABuiltInType.Boolean),
                new UAFieldMetaData("Byte", UABuiltInType.Byte),
                new UAFieldMetaData("Int16", UABuiltInType.Int16),
                new UAFieldMetaData("Int32", UABuiltInType.Int32),
                new UAFieldMetaData("SByte", UABuiltInType.SByte),
                new UAFieldMetaData("UInt16", UABuiltInType.UInt16),
                new UAFieldMetaData("UInt32", UABuiltInType.UInt32),
                new UAFieldMetaData("Float", UABuiltInType.Float),
                new UAFieldMetaData("Double", UABuiltInType.Double)
            });
            var subscribeDataSetArguments2 = new EasyUASubscribeDataSetArguments(
                connectionDescriptor, subscribeDataSetFilter, dataSetMetaData2)
            {
                DataSetSubscriptionDescriptor = { CommunicationParameters =
                {
                    MessageReceiveTimeout = 6000,
                    UadpDataSetReaderMessageParameters = { DataSetOffset = 37 }
                }}
            };

            //

            var fields = new UAFieldMetaDataCollection();
            for (int i = 0; i < 100; i++)
                fields.Add(new UAFieldMetaData(
                    FormattableString.Invariant($"Mass_{i}"),
                    UABuiltInType.UInt32));
            var dataSetMetaData3 = new UADataSetMetaData("MassTest", fields);
            var subscribeDataSetArguments3 = new EasyUASubscribeDataSetArguments(
                connectionDescriptor, subscribeDataSetFilter, dataSetMetaData3)
            {
                DataSetSubscriptionDescriptor = { CommunicationParameters =
                {
                    MessageReceiveTimeout = 6000,
                    UadpDataSetReaderMessageParameters = { DataSetOffset = 69 }
                }}
            };

            //

            subscriber.SubscribeDataSet(subscribeDataSetArguments1);
            subscriber.SubscribeDataSet(subscribeDataSetArguments2);
            subscriber.SubscribeDataSet(subscribeDataSetArguments3);

            AssertUtilities.WaitingPhase(2 * 60 * 1000);


        }
    }
}
